package com.qmth.sop.business.templete.service.impl;

import com.alibaba.excel.EasyExcel;
import com.qmth.sop.business.bean.dto.SysUserImportDto;
import com.qmth.sop.business.entity.TBTask;
import com.qmth.sop.business.templete.service.TaskLogicService;
import com.qmth.sop.business.util.ImportExportUtil;
import com.qmth.sop.business.util.excel.BasicExcelListener;
import com.qmth.sop.common.contant.SystemConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    ImportExportUtil importExportUtil;

    /**
     * 处理导入用户数据
     *
     * @param map 数据源
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public Map<String, Object> executeImportSysUserLogic(Map<String, Object> map) throws Exception {
        InputStream inputStream = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            inputStream = importExportUtil.getUploadFileInputStream(tbTask);
            LinkedMultiValueMap<String, SysUserImportDto> sysUserImportDtoLinkedMultiValueMap = new LinkedMultiValueMap<>();
            StringJoiner errorData = new StringJoiner("");
            EasyExcel.read(inputStream, SysUserImportDto.class, new BasicExcelListener<SysUserImportDto>() {

                @Override
                public void handle(LinkedMultiValueMap<String, SysUserImportDto> dataList, StringJoiner errorDataSj) {
                    sysUserImportDtoLinkedMultiValueMap.addAll(dataList);
                    errorData.add(errorDataSj.toString());
                }
            }).headRowNumber(2).sheet(0).doRead();

            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> sysUserImportDtoLinkedMultiValueMap);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA_ERROR, v -> errorData.toString());

            //处理数据逻辑
        } finally {
            if (Objects.nonNull(inputStream)) {
                inputStream.close();
            }
        }
        return map;
    }
}
